%This file contains the source code for the Nordic SMC winter school
%session on Mocap Toolbox. It's organised in sections; each section can be
%run individually by pressing the "run section" button in the menu above,
%or by pressing ctrl+Enter (CMD+Enter for mac)

%% Reading a mocap file:

a = mcread('Salsa_SD0001.mat') 

% 'a' is now a struct
% important fields in the struct
%    - nFrames
%    - freq
%    - nMarkers
%    - data
%    - markerName
%    - timederOrder
%    - type
%
%   Also have a look at the variable editor window. Double click on 'a' in
%   the workspace.
%    - double click on 'markerName' to see the cell array with marker names
%    - double click on 'data' to see the data matrix


%% Plot frame number 3333 of the mocap recording

mcplotframe(a,3333)


%% plot frame number 3200, 3225, 3250, ... , 3400 of the mocap recording
mcplotframe(a,3200:25:3400)


%% let's close all the figure windows 
close all

%% Check out Matlab's help function
% The help window gives more information: 
% type mcplotframe (without pressing return) in the command window and 
% then press F1.
%
% The help file tells us that the animation can be changed with an 
% 'animpar structure'

%% Generate an animpar structure:
p = mcinitanimpar

% some of the useful fields in the animpar structure:
%   - colors:       change the color appearance of the plot
%   - msize:        change marker size
%   - conn:         set up connecting lines between markers
%   - animate:      set to 1 to make a video file
%   - videoformat:  mpeg or avi format
%   - output:       name of video file

%% Let's adjust the size of the markers and the colors
p.msize = 16;
p.colors = 'wbbbb';

%and then plot the same plot again, with the animpar structure as input:
mcplotframe(a,3333,p)



%% Add a few connections to the animation parameter struct
% find the relevant markers in a.markerName

p.conn = [7 8;8 9; 9 10]; %connecting right shoulder - elbow - wrist - hand 
p.cwidth = 3;             %adjusting the width of the connection lines

mcplotframe(a,3300,p) 

%% Load a pre-made animation parameter struct, and look at the result: 

load animpar.mat

mcplotframe(a,3200:25:3400,animpar) 

%% close the figure windows
close all


%%
% Let's have a closer look at the different types of data and support
% structures in the MoCap toolbox:
% - 'MoCap data'
% - 'norm data'
% - 'segm data'
% 
% - 'animpar'
% - 'm2jpar'
% - 'j2spar'


%% More on plotting. Let's plot marker number 1 on time line
mcplottimeseries(a,1)
 
%% And specifically plot dimension number 3 of marker 1 (vertical position)
mcplottimeseries(a,1,'dim',3)

%% Plot Left Front Hip and Right Front Hip markers, both X, Y, and Z pos.
mcplottimeseries(a,{'LFHIP','RFHIP'},'dim',1:3)

%% Plot the same in a combined plot
clf %clear figure
mcplottimeseries(a,{'LFHIP','RFHIP'},'dim',3,'plotopt','comb')
 

%% Perhaps better to only work with a short segment of the data:

a2 = mctrim(a,40,50)
mcplottimeseries(a2,{'LFHIP','RFHIP'},'dim',3,'plotopt','comb')

%let's start working with joint data instead of marker data
%% Converting marker data to joint data: First initialize a marker-to-joint struct

p = mcinitm2jpar

%% Set up combinations of markers in a cell array:
 
%collapse markers 1-4 into a single "head" marker, and 5-6 a single 'Neck'
%marker. Keep markers 7, 8, 9, 10 as they were:
p.markerNum = {[1 2 3 4],[5 6],[7],8,9,10};
p.markerName = {'Head','Neck','RSHO','RELB','RWRI','RHAN'};
p.nMarkers = 6 %We've only included 6 markers so far

%% assign the new joint data to the variable aj and plot.
aj = mcm2j(a,p)
mcplotframe(aj,3300)

 
%% close figures
close all

%% load a premade marker to joint parameter structure:
load m2jpar.mat
aj = mcm2j(a,m2jpar)
mcplotframe(aj,3300)

%% We cannot use the old animpar struct anymore, because we have a different marker set
%We update the connections in our animpar structure:
animpar.conn = [2 3;3 4;4 5;5 6;7 8;8 9;9 10;10 11;1 12;12 13;13 14; 13 15;15 16;16 17;17 18;13 19;19 20;20 21;21 22;1 2;1 7];
mcplotframe(aj,5000,animpar)

%% Estimate the first and second time derivatives of the data (i.e. velocity and acceleration)
 ajd1 = mctimeder(aj,1)
 ajd2 = mctimeder(aj,2)


%% Plot the head marker

close all
figure, mcplottimeseries(aj,{'head','lfinger'},'dim',1:3)
figure, mcplottimeseries(ajd1,{'head','lfinger'},'dim',1:3)
figure, mcplottimeseries(ajd2,{'head','lfinger'},'dim',1:3)

%% Let's convert the data to metres to get SI units

aj.data = aj.data * 0.001;  %was millimetres, so now it's metres
ajd1 = mctimeder(aj,1);     %this one is now metres per second
ajd2 = mctimeder(aj,2);     %and this one is metres per squared second

% just for fun, let's change the acceleration data unit from metres per 
% squared second to gravity units
ajd2.data = ajd2.data*0.980665;

close all
figure, mcplottimeseries(aj,{'head','lfinger'},'dim',1:3)
figure, mcplottimeseries(ajd1,{'head','lfinger'},'dim',1:3)
figure, mcplottimeseries(ajd2,{'head','lfinger'},'dim',1:3)

%or even
 ajd2.data = ajd2.data*0.980665; %to get acc in gravity unit
 mcplottimeseries(ajd2,{'head','lfinger'},'dim',1:3)

 
%% Sometimes we're rather interested in magnitude than in 3D / X-Y-Z data

ajd1n = mcnorm(ajd1)
ajd2n = mcnorm(ajd2)
%notice: 'norm struct' type

close all
mcplottimeseries(ajd1n,{'head','lfinger'})


%% So far we've worked with nice and clean data. Let's load some noisy data
load aWithGapsAndNoise.mat

%report the number of missing frames for each marker
mcmissing(aWithGapsAndNoise)

%plot marker 15, which is both noisy and full of gaps.
mcplottimeseries(aWithGapsAndNoise,15,'dim',1)

%% Use mcsmoothen to smooth the data
asmooth = mcsmoothen(aWithGapsAndNoise)
mcplottimeseries(asmooth,15,'dim',1)

%% Give an additional argument to mcsmoothen to adjust the amount of smoothing
asmooth = mcsmoothen(aWithGapsAndNoise,25)
mcplottimeseries(asmooth,15,'dim',1)

%% Fill the faps in the data using mcfillgaps
 afilled = mcfillgaps(asmooth)
 mcplottimeseries(afilled,15,'dim',1)
 hold all,mcplottimeseries(asmooth,15,'dim',1)
% the resulting plot show the filled data together with the data with gaps

%% Set a maximum gap length to avoid filling "impossible" gaps
 afilled = mcfillgaps(asmooth,100) %filling gaps up to 0.5 seconds – even this is probably a bit too much
clf, mcplottimeseries(afilled,15,'dim',1)
 hold all,mcplottimeseries(asmooth,15,'dim',1)



%% Some other useful functions
%- mcdemo: nice demos of several mocap toolbox functions
%- mc2frontal: Rotates the data to always face in a certain direction,
%              either frame-by-frame or by mean value.
%- mcmarkerdist: calculates the distance between two markers
%- mcmean & mcstd: Mean and standard deviation of data matrix (over time)
%- mcconcatenate: combine selected markers form one or more mocap structs into a new mocap struct
%- mccumdist: cumulative distance traveled for a marker
%- mcmerge: merges the marker sets and animation parameters from two mocap structs
%- mcpcaproj: Principal Component Analysis for mocap data
%- mcicaproj: Independent Component Analysis for mocap data
%- mcperiod: periodicity analysis with autocorrelation
%- mcrotate: rotate the mocap data
%- mctranslate: translate the mocap data
%- mcwindow: windowed analysis
%- mckinenergy: estimate the instantaneous kinetic energy of body segments
%- mcpotenergy: estimate the instantaneous potential energy of body segments
%- mapping of joints to bodysegments

%% Additions, available at https://github.com/krisny/MusicMocapMatlab/
%
%- mcmeanbar, calculate average movement of a marker across a fixed bar length
%- mcplot3Dframe, plot a mocap frame in 3D
%- mcanimate3D, create a 3D animation of the mocap data
%- mcappend, append one mocap struct after another in time
 

